/*
 * @file xaxivdma_example_selftest.c
 *
 * <pre>
 * MODIFICATION HISTORY:
 *
 *
 */

/***************************** Include Files *********************************/
#include "main.h"

/************************** Variable Definitions *****************************/
/*
 * Device instance definitions
 */
XGpio          Gpio;          /* The Instance of the GPIO 0 Driver */

u32 frame_read[6] = {FRAME_R_BASE_1,FRAME_R_BASE_2,FRAME_R_BASE_3,FRAME_R_BASE_1,FRAME_R_BASE_2,FRAME_R_BASE_3};



/*****************************************************************************/
/**
* The entry point for the WIDI project. It initializes and configures 
* the devices then starts the 60Hz timer and reports the execution status.
*
* @param    None.
*
* @return
*     - XST_SUCCESS if successful
*     - XST_FAILURE if fails.
*
* @note     None.
*
******************************************************************************/
int main()
{
   int Status;

   FrameBufferNum = 0;

   WriteMem(testFrame[0], FRAME_WORDS_X3, 0xFFFFFFFF);

   WriteMem(testFrame[1], FRAME_WORDS_X3, 0xF0F0F0F0);


   Status = RxConfig();

   if (Status != XST_SUCCESS) {
	  xil_printf("RX Configuration function failed \r\n");
	  return XST_FAILURE;
   }

   //Flush cache
   Xil_DCacheFlush();

   //Start the timer counter
   XTtcPs_Start(&Timer);

   for (;;){

   }


   return XST_SUCCESS;

}


/*****************************************************************************/
/**
* This function configures the receiver devices: GPIO & VDMA Read Channel
*
* @param    None.
*
* @return
*     - XST_SUCCESS if successful
*     - XST_FAILURE if fails.
*
* @note     None.
*
******************************************************************************/
int RxConfig()
{
   int Status;

   Status = XGpio_Initialize(&Gpio, GPIO_DEV_ID);
   if (Status != XST_SUCCESS) {
	  xil_printf("Gpio Initialization Failed\r\n");
	  return XST_FAILURE;
   }

   /* Run the poll example for simple transfer */
   Status = AxiVDMASelfTestExample(&AxiVdma,DMA_DEV_ID);

   if (Status != XST_SUCCESS) {

	  xil_printf("AxiVDMASelfTest Example Failed\r\n");
	  return XST_FAILURE;
   }

   xil_printf("Successfully ran Read AxiVDMASelfTest Example\r\n");

   Status = TimerInit(&Timer, &Intc, TIMER_DEV_ID, TIMER_IRPT);

	if (Status != (XST_SUCCESS)) {
		xil_printf("Timer Init Failed.\r\n");
		return (XST_FAILURE);
	}

   //Configure VDMA Read
   Status = configureRead(H_RES, V_RES);

   if (Status != (XST_SUCCESS)) {
	  xil_printf("VDMA Configure Read Failed.\r\n");
	  return (XST_FAILURE);
   }

   //Enable VDMA Read
   Status = XAxiVdma_DmaStart(&AxiVdma, XAXIVDMA_READ);

   if (Status != (XST_SUCCESS)) {
	  xil_printf("VDMA Enable Read Failed.\r\n");
	  return (XST_FAILURE);
   }



   return XST_SUCCESS;
}

/*****************************************************************************/
/**
* This function handles the timer interrupt function. It copys memory from
* VDMA write channel DDR memory to the VDMA read channel DDR memory block.
* Currently only used by the TX side of the project
*
* @param    None.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void IRQHanlder(void *CallBackRef)
{
   u32 *r_addr;
   u32 *w_addr;
   u32 g_code_rd;
   u32 n_rd;

   u32 StatusEvent;

   
   if (FrameBufferNum == 0) {
	   FrameBufferNum++;
   } else {
	   FrameBufferNum--;
   }

   /*
    * Read the interrupt status, then write it back to clear the interrupt.
    */
   StatusEvent = XTtcPs_GetInterruptStatus((XTtcPs *)CallBackRef);
   XTtcPs_ClearInterruptStatus((XTtcPs *)CallBackRef, StatusEvent);

   g_code_rd = XGpio_DiscreteRead(&Gpio,PNTR_CHANNEL);

   n_rd = grayCode(g_code_rd) - 1;

   if (n_rd == 0) {
      n_rd = 6;
   }

   r_addr = (u32*)frame_read[n_rd - 1];
   w_addr = testFrame[!FrameBufferNum];

   Xil_DCacheInvalidateRange((u32)r_addr,FRAME_WORDS_X3);
   Xil_MemCpy(r_addr,w_addr,FRAME_WORDS_X3);
   Xil_DCacheFlushRange((u32)r_addr,FRAME_WORDS_X3);


   xil_printf("--- IRQ \r\n");


}


/*****************************************************************************/
/**
* This function performs a memory read and prints the results.
*
* @param    *Addr: start of address to be read
*           Words: 32 bit length of 32 bit words to read
*
* @return
*     - XST_SUCCESS if channel reset is successful
*     - XST_FAILURE if channel reset fails.
*
* @note     None.
*
******************************************************************************/
void ReadMem(u32 *Addr, u32 Words)
{
   u32 J = 6;
   u32 I;
   u32 WordMem;

   /*
    * Read third frame buffer
    */
   for (I = 0U; I < Words; I++) {
       WordMem = *(Addr+I);
       xil_printf("0x%08x | ", WordMem);
       if ((I % J) == 0) {
          xil_printf("\r\n");
       }
   }
}

/*****************************************************************************/
/**
* This function performs a memory wirte.
*
* @param    *Addr: start of address to write to
*           Words: 32 bit length of 32 bit words to write
*           Data: 32 bit data value to write
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void WriteMem(u32 *Addr, u32 Words, u32 Data)
{
   u32 I;
   u32 Val;
   /*
    * variable initialization
    */
   Val = Data;

   for (I = 0U; I < Words; I++) {
      *(Addr+I) = Val;
   }

}

/*****************************************************************************/
/**
* This function converts gray codes to decimal values.
*
* @param    n: gray code
*
* @return   None.
*
* @note     Integer decimal value
*
******************************************************************************/
int grayCode(u32 n)
{
    int inv = 0; 
  
    // Taking xor until n becomes zero 
    for (; n; n = n >> 1) 
        inv ^= n; 
  
    return inv;
}
