
/***************************** Include Files *********************************/
#include "main.h"

static TmrCntrSetup SettingsTable = {
			60, 0, 0, 0	/* Ticker timer counter initial setup, only output freq */
};


/*****************************************************************************/
/**
* This function initializes both the timer and interrupt for the 60 Hz timer
*
* @param TimerPtr is a pointer to the XScuTimer device
*        IntcPtr is a point to the XScuGic device
*        TimerId is the device ID of the timer 
*        TimerIntrId is the ID of the interrupt mapped to the timer
*
* @return
*     - XST_SUCCESS if init is successful
*     - XST_FAILURE if init fails.
*
* @note     None.
*
******************************************************************************/
int TimerInit(XTtcPs *TimerPtr, XScuGic *IntcPtr, u16 TimerId, u16 TimerIntrId){
   XStatus status;

   XTtcPs_Config *ConfigPtr;
   XScuGic_Config *IntcConfig;
   TmrCntrSetup *TimerSetup;

   TimerSetup = &SettingsTable;

   TimerSetup->Options |= (XTTCPS_OPTION_INTERVAL_MODE |
					      XTTCPS_OPTION_WAVE_DISABLE);

   /*
    * Initialize the Scu Private Timer driver.
    */
   ConfigPtr = XTtcPs_LookupConfig(TimerId);

   /*
    * This is where the virtual address would be used, this example
    * uses physical address.
    */
   status = XTtcPs_CfgInitialize(TimerPtr, ConfigPtr,
               ConfigPtr->BaseAddress);
   if (status != XST_SUCCESS) {
      return XST_FAILURE;
   }

   /*
    * Perform a self-test to ensure that the hardware was built correctly.
    */
   status = XTtcPs_SelfTest(TimerPtr);
   if (status != XST_SUCCESS) {
      return XST_FAILURE;
   }

   XTtcPs_SetOptions(TimerPtr,TimerSetup->Options);

	/*
	 * Timer frequency is preset in the TimerSetup structure,
	 * however, the value is not reflected in its other fields, such as
	 * IntervalValue and PrescalerValue. The following call will map the
	 * frequency to the interval and prescaler values.
	 */
	XTtcPs_CalcIntervalFromFreq(TimerPtr, TimerSetup->OutputHz,
		&(TimerSetup->Interval), &(TimerSetup->Prescaler));

	/*
	 * Set the interval and prescale
	 */
	XTtcPs_SetInterval(TimerPtr, TimerSetup->Interval);
	XTtcPs_SetPrescaler(TimerPtr, TimerSetup->Prescaler);

   /*
    * Connect the device to interrupt subsystem so that interrupts
    * can occur.
    */
   IntcConfig = XScuGic_LookupConfig(INTC_DEV_ID);
   if (NULL == IntcConfig) {
      return XST_FAILURE;
   }

   status = XScuGic_CfgInitialize(IntcPtr, IntcConfig,
               IntcConfig->CpuBaseAddress);
   if (status != XST_SUCCESS) {
      return XST_FAILURE;
   }

   Xil_ExceptionInit();

   /*
    * Connect the interrupt controller interrupt handler to the hardware
    * interrupt handling logic in the processor.
    */
   Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_IRQ_INT,
            (Xil_ExceptionHandler)XScuGic_InterruptHandler,
            IntcPtr);

   /*
    * Connect the device driver handler that will be called when an
    * interrupt for the device occurs, the handler defined above performs
    * the specific interrupt processing for the device.
    */
   status = XScuGic_Connect(IntcPtr, TimerIntrId,
            (Xil_InterruptHandler)IRQHanlder,
            (void *)TimerPtr);
   if (status != XST_SUCCESS) {
      return XST_FAILURE;
   }

   /*
    * Enable the interrupt for the device.
    */
   XScuGic_Enable(IntcPtr, TimerIntrId);

   /*
    * Enable the timer interrupts for timer mode.
    */
   XTtcPs_EnableInterrupts(TimerPtr, XTTCPS_IXR_INTERVAL_MASK);

   /*
    * Enable interrupts in the Processor.
    */
   Xil_ExceptionEnable();


   return status;
}
